game = Game:addState "game"

-- local variables
local enemies
local enemy_n
local player
local target

local epilepsy_colors = {
	Color:new(000, 000, 000);
	Color:new(255, 255, 255);
}
local epilepsy_it = 1
local last_blink = love.timer.getTime()

local directions = {
	up = Vector:new(0, -1),
	down = Vector:new(0, 1),
	right = Vector:new(1, 0),
	left = Vector:new(-1, 0)
}

function game:newEnemy()
	if enemy_n > 0 then enemies[enemy_n].spawning = false end
	enemy_n = enemy_n + 1
	enemies[enemy_n] = Enemy:new()
end

function game:enterState()
	enemies = {}
	enemy_n = 0
	player = Player:new()
	target = Target:new()
	self:newEnemy()
	if self.mode == "epilepsy" then
		epilepsy_colors[1]:setBackground()
	end
	self.score = 0
	math.randomseed(os.time())
	math.random() math.random() math.random()
end

function game:update(dt)
	if self.mode == "extreme" then
		dt = dt*2
	end
	for n=1,enemy_n do
		enemies[n]:update(dt)
		if not enemies[n].spawning and enemies[n]:colliding(player) then
			if self.mode == "epilepsy" and last_blink+0.05 < love.timer.getTime() then
				epilepsy_it = epilepsy_it + 1
				if epilepsy_it == #epilepsy_colors + 1 then
					epilepsy_it = 1
				end
				epilepsy_colors[epilepsy_it]:setBackground()
				last_blink = love.timer.getTime()
			end
			player:modHealth(-25*dt)
		end
	end
	local dir = Vector:new(love.joystick.getAxis(0, 0), love.joystick.getAxis(0, 1))
	player:move(dir * dt * 150)
	if target:colliding(player) then
		player:modHealth(1)
		game:newEnemy()
		target:newPos()
		self.score = self.score + 1
	end
	if player.hp == 0 or (self.mode == "insta-kill" and player.hp ~= 100) then
		self:gotoState "over"
	end
end

function game:draw()
	for n=1,enemy_n do
		enemies[n]:draw()
	end
	player:draw()
	target:draw()
	if self.mode == "epilepsy" then
		epilepsy_colors[epilepsy_it]:invert():set()
	else
		love.graphics.setColor(255, 255, 255)
	end
	love.graphics.print("health: "..math.ceil(player.hp), 5, 20)
	love.graphics.print("score: "..self.score, 5, 35)
end

function game:jp(_, key)
	if key == 8 then
		player.hp = 0
	end
end
